# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from rest_framework.authentication import BaseAuthentication
from rest_framework.authentication import get_authorization_header
from rest_framework import exceptions
from rest_framework import HTTP_HEADER_ENCODING
from social_core.exceptions import MissingBackend
from social_core.exceptions import AuthForbidden
from social_core.utils import requests
from social_django.views import NAMESPACE
from social_django.utils import load_backend
from social_django.utils import load_strategy


class SocialAuthentication(BaseAuthentication):
    """
    `python-social-auth` based django rest framework authentication.

    Clients authenticate using the token key in the "Authorization"
    HTTP header. For example:

        Authorization: Bearer g87ewgbgg78wegpngxgfoawgbo78aegnafxngfe
    """
    www_authenticate_realm = 'api'

    def authenticate(self, request):
        """
        Authenticate the request and return a two-tuple of (user, token).
        """
        auth_header = get_authorization_header(request).decode(HTTP_HEADER_ENCODING)

        auth_parts = auth_header.split()

        if len(auth_parts) == 1:
            msg = 'Invalid token header. Credentials not provided.'
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth_parts) > 2:
            msg = 'Invalid token header. The token string cannot contain spaces.'
            raise exceptions.AuthenticationFailed(msg)

        auth_type, auth_token = auth_parts[0], auth_parts[1]

        if auth_type.lower() != 'bearer':
            msg = 'Invalid authentication header type. Only Bearer tokens are currently supported..'
            raise exceptions.AuthenticationFailed(msg)

        strategy = load_strategy(request=request)

        try:
            backend = load_backend(strategy, 'auth0', reverse(NAMESPACE + ":complete", args=('auth0',)))
        except MissingBackend:
            msg = 'Either token header is invalid or the backend could not be loaded.'
            raise exceptions.AuthenticationFailed(msg)

        try:
            user = backend.do_auth(access_token=auth_token)
        except AuthForbidden as err:
            raise exceptions.AuthenticationFailed(err)
        except requests.HTTPError as e:
            msg = e.response.text
            raise exceptions.AuthenticationFailed(msg)

        if not user:
            msg = 'Unable to authenticate these credentials.'
            raise exceptions.AuthenticationFailed(msg)
        return user, auth_token

    def authenticate_header(self, request):
        """
        Return a string to be used as the value of the `WWW-Authenticate`
        header in a `401 Unauthenticated` response, or `None` if the
        authentication scheme should return `403 Permission Denied` responses.
        """
        return 'Bearer realm="%s"' % self.www_authenticate_realm



import logging
import random
from time import sleep

from auth0.v3.exceptions import Auth0Error
from auth0.v3.management import Auth0
from django.conf import settings
from elizabeth import Personal
from retryz import retry

from django_auth0_user.auth0.api import AUTH0_TOKEN_CACHE, get_auth0, get_users_from_auth0, get_auth0_user

logger = logging.getLogger(__name__)


person = Personal('en')


@retry(wait=5, timeout=300, on_return=False)
def confirm_user_total(desired_total, auth0=None):
    if auth0 is None:
        auth0 = get_auth0()
    total_users = len(list(get_users_from_auth0(auth0)))
    return total_users == desired_total


def create_auth0_test_users(number_of_users, overrides=None, auth0=None):
    if auth0 is None:
        auth0 = get_auth0()
    auth0_users = []
    for i in range(number_of_users):
        gender = random.choice(['male', 'female'])
        test_user_details = {
            "connection": "Username-Password-Authentication",
            # "username": person.username(gender=gender),
            "password": person.password(length=8),
            "email": person.email(gender=gender, domains=['@example.com', ]),
            "email_verified": False,
            "verify_email": False,
            # "phone_number": person.telephone(mask="+############"),
            # "phone_verified": False,
            "user_metadata": {},
            "app_metadata": {},
        }
        if overrides is not None:
            test_user_details.update(overrides)
        user = auth0.users.create(body=test_user_details)
        logger.info("generated a new auth0 user: {}".format(user))
        # confirming user can be read
        retrieved_user = auth0.users.get(user['user_id'])
        auth0_users.append(retrieved_user)
        logger.info("Retrieved new auth0 user: {}".format(retrieved_user))


@retry(wait=5, timeout=300, on_return=False)
def create_and_confirm_auth0_test_users(desired_total, auth0=None):
    if auth0 is None:
        auth0 = get_auth0()
    current_user_total = len(list(get_users_from_auth0(auth0)))
    difference = desired_total - current_user_total
    if difference > 0:
        for i in range(difference):
            gender = random.choice(['male', 'female'])
            test_user_details = {
                "connection": "Username-Password-Authentication",
                # "username": person.username(gender=gender),
                "password": person.password(length=8),
                "email": person.email(gender=gender, domains=['@example.com', ]),
                "email_verified": False,
                "verify_email": False,
                # "phone_number": person.telephone(mask="+############"),
                # "phone_verified": False,
                "user_metadata": {},
                "app_metadata": {},
            }
            user = auth0.users.create(body=test_user_details)
            logger.info("generated a new auth0 user: {}".format(user))

    total_users = len(list(get_users_from_auth0(auth0)))
    return total_users == desired_total


def create_auth0_user(auth0=None, overrides=None):
    if auth0 is None:
        auth0 = get_auth0()
    gender = random.choice(['male', 'female'])
    test_user_details = {
        "connection": "Username-Password-Authentication",
        # "username": person.username(gender=gender),
        "password": person.password(length=8),
        "email": person.email(gender=gender, domains=['@example.com', ]),
        "email_verified": False,
        "verify_email": False,
        # "phone_number": person.telephone(mask="+############"),
        # "phone_verified": False,
        "user_metadata": {},
        "app_metadata": {},
    }
    if overrides is not None:
        test_user_details.update(overrides)
    user = auth0.users.create(body=test_user_details)
    logger.info("generated a new auth0 user: {}".format(user))


def create_ten_auth0_users():
    auth0 = Auth0(settings.AUTH0_USER_DOMAIN, AUTH0_TOKEN_CACHE.auth0_management_api_token)
    for i in range(10):
        create_auth0_user(auth0)


def delete_all_auth0_users():
    auth0 = Auth0(settings.AUTH0_USER_DOMAIN, AUTH0_TOKEN_CACHE.auth0_management_api_token)
    all_auth0_users = list(get_users_from_auth0(auth0))
    while len(list(get_users_from_auth0(auth0))) > 0:
        for user in all_auth0_users:
            auth0.users.delete(user['user_id'])

    check_empty = list(get_users_from_auth0(auth0))
    logger.info("{} users remain.".format(len(check_empty)))


def delete_auth0_user_and_confirm(user, auth0=None):
    if auth0 is None:
        auth0 = get_auth0()

    user_id = user['user_id']
    deletion_response = auth0.users.delete(user_id)
    logger.info("Deletion Response: {}".format(deletion_response))

    @retry(wait=5, timeout=300, on_return=False)
    def confirm(target_user_id):
        try:
            query_response = auth0.users.get(target_user_id)
            logger.info("Found User: {}".format(query_response))
        except Auth0Error as err:
            if str(err) == "404: The user does not exist.":
                logger.info("Response: {}".format(err))
                return True
        return False

    confirm(user_id)


# @retry(wait=1, timeout=30, on_return=False)
def delete_all_auth0_users_with_confirmation():
    logger.info("Attempting to delete all auth0 users, with confirmation.")
    auth0 = get_auth0()
    all_auth0_users = list(get_users_from_auth0(auth0))
    for user in all_auth0_users:
        delete_auth0_user_and_confirm(user, auth0=auth0)
        logger.info("Deleted: {}".format(user['user_id']))
    total_users = len(list(get_users_from_auth0(auth0)))
    logger.info("{} users remain.".format(total_users))
    return total_users == 0


def create_auth0_user_and_confirm(auth0=None):
    if auth0 is None:
        auth0 = get_auth0()

    gender = random.choice(['male', 'female'])
    test_user_details = {
        "connection": "Username-Password-Authentication",
        # "username": person.username(gender=gender),
        "password": person.password(length=8),
        "email": person.email(gender=gender, domains=['@example.com', ]),
        "email_verified": False,
        "verify_email": False,
        # "phone_number": person.telephone(mask="+############"),
        # "phone_verified": False,
        "user_metadata": {},
        "app_metadata": {},
    }
    user = auth0.users.create(body=test_user_details)
    logger.info("generated a new auth0 user: {}".format(user))

    @retry(wait=5, timeout=300, on_return=False)
    def confirm(target_user_id):
        try:
            query_response = auth0.users.get(target_user_id)
            logger.info("Found User: {}".format(query_response))
            return True
        except Auth0Error as err:
            logger.info("Response: {}".format(err))
            return False

    confirm(user['user_id'])


def create_multiple_auth0_users_and_confirm(number_of_users_to_create, auth0=None):
    logger.info("Attempting to create {} auth0 users, with confirmation.".format(number_of_users_to_create))
    if auth0 is None:
        auth0 = get_auth0()

    user_list = []
    for i in range(number_of_users_to_create):
        gender = random.choice(['male', 'female'])
        test_user_details = {
            "connection": "Username-Password-Authentication",
            # "username": person.username(gender=gender),
            "password": person.password(length=8),
            "email": person.email(gender=gender, domains=['@example.com', ]),
            "email_verified": False,
            "verify_email": False,
            # "phone_number": person.telephone(mask="+############"),
            # "phone_verified": False,
            "user_metadata": {},
            "app_metadata": {},
        }
        user = auth0.users.create(body=test_user_details)
        logger.info("generated a new auth0 user: {}".format(user))
        user_list.append({**test_user_details, **user})

    # TODO: Pass in a list of users and query to check they are all in auth0, retry until they are all in auth0.
    @retry(wait=5, timeout=300, on_return=False)
    def confirm(_user_list):
        _user_ids = [_x['user_id'] for _x in _user_list]
        _retrieved_user_list = []
        _retrieved_user_ids = []
        for _user in _user_list:
            _auth0_user = get_auth0_user(_user['user_id'])
            _retrieved_user_ids.append(_auth0_user['user_id'])
            _retrieved_user_list.append(_auth0_user)

        if set(_user_ids) == set(_retrieved_user_ids):
            logger.info("Found all new users users. {}".format(_user_ids))
            return True
        return False

    confirm(user_list)
    return user_list


def pause_and_confirm_total_auth0_users(pause_duration, desired_total_users, auth0=None):
    if auth0 is None:
        auth0 = get_auth0()
    logger.info('Pausing for {} seconds'.format(pause_duration))
    sleep(pause_duration)
    logger.info('Confirming {} Auth0 users'.format(desired_total_users))

    @retry(wait=1, timeout=60, on_return=False)
    def confirm(desired_total):
        number_of_users = len(list(get_users_from_auth0(auth0)))
        logger.info("Found {} users.".format(number_of_users))
        if number_of_users == desired_total:
            return True
        return False

    confirm(desired_total_users)




from django.test.selenium import SeleniumTestCase
from selenium.webdriver.support.ui import WebDriverWait
# noinspection PyPep8Naming
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
import pytest
# noinspection PyUnresolvedReferences,PyPackageRequirements
from tests.utils.fixtures import one_auth0_user


# TODO: Convert this into a non-class based test so we can drive it more flexibly via PyTest.
@pytest.mark.usefixtures('one_auth0_user')
class WebSiteAuthFlow(SeleniumTestCase):
    """
    Integration test of Auth0 web authentication process.

    The PyTest fixture generates a new random user on Auth0 and
    """
    browser = 'firefox'
    port = 63180
    # browser = 'phantomjs'
    # browsers = ['firefox']

    # def setUp(self):
    #     delete_all_auth0_users_with_confirmation()
    #     create_auth0_user(overrides={'email': 'user1@example.com', 'password': 'known_password'})
    #     create_auth0_user(overrides={'email': 'user2@example.com', 'password': 'known_password'})

    def test_regular_user_login(self):
        # """Self.user is
        user_email = self.user['email']
        user_password = self.user['password']

        self.selenium.get('%s%s' % (self.live_server_url, '/'))
        self.selenium.find_element_by_link_text('Login').click()
        wait = WebDriverWait(self.selenium, 60)
        wait.until(EC.visibility_of_element_located(
            (By.XPATH, '//div[@class="auth0-lock-input-block auth0-lock-input-email"]/div/input')))
        self.selenium.find_element_by_xpath(
            '//div[@class="auth0-lock-input-block auth0-lock-input-email"]/div/input').send_keys(user_email)
        self.selenium.find_element_by_xpath(
            '//div[@class="auth0-lock-input-block auth0-lock-input-password"]/div/input').send_keys(user_password)
        self.selenium.find_element_by_xpath('//button[@class="auth0-lock-submit"]').click()
        email_on_page = self.selenium.find_element_by_xpath('//span[@id="user-email"]').text
        assert email_on_page == user_email
        # TODO: Assert something about the username being the auth0 user_id.



from test_app.models import Auth0User
import logging


logger = logging.getLogger(__name__)


def delete_all_django_users():
    logger.info("Attempting to delete all Django users.")
    user_list = list(Auth0User.objects.all())
    logger.info('Found {} existing django users.'.format(len(user_list)))
    for user in user_list:
        user.delete()



from tests.utils.auth0 import delete_all_auth0_users as delete_all_auth0_users_via_api
from tests.utils.auth0 import create_auth0_test_users
from tests.utils.auth0 import create_and_confirm_auth0_test_users
from tests.utils.auth0 import delete_all_auth0_users_with_confirmation
from tests.utils.auth0 import create_multiple_auth0_users_and_confirm
from tests.utils.auth0 import pause_and_confirm_total_auth0_users
from tests.utils.django import delete_all_django_users
from test_app.models import Auth0User
import pytest
import logging


logger = logging.getLogger(__name__)


DELAY = 15


@pytest.fixture(scope="class")
def one_auth0_user(request):
    """
    Pytest fixture providing one Auth0 user for testing.

    Create a new user in Auth0 and add it to the class of our requesting test case class at runtime.
    This is a little different than most PyTest fixtures because are working around the fact that
    we are using SeleniumTestCase for some tests, which as a subclass of TestCase cannot use normal
    PyTest fixture based parametrization.

    :param request:
    :return:
    """
    users = create_multiple_auth0_users_and_confirm(1)
    request.cls.user = users[0]


@pytest.fixture(scope='function')
def delete_all_auth0_users():
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_via_api()
    yield
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_via_api()


@pytest.fixture(scope='function')
def delete_all_django_users():
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    yield
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()


@pytest.fixture(scope='function')
def cleanup_django_and_auth0():
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_via_api()
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    yield
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_via_api()


@pytest.fixture(scope='function')
def one_user():
    number_of_users = 1
    logger.info('Start of one_user() fixture.')
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_with_confirmation()
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    create_multiple_auth0_users_and_confirm(number_of_users)
    pause_and_confirm_total_auth0_users(DELAY, number_of_users)
    yield
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_with_confirmation()
    logger.info('End of one_user() fixture.')


@pytest.fixture(scope='function')
def five_users():
    number_of_users = 5
    logger.info('Start of five_users() fixture.')
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_with_confirmation()
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    create_multiple_auth0_users_and_confirm(number_of_users)
    pause_and_confirm_total_auth0_users(DELAY, number_of_users)
    yield
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_with_confirmation()
    logger.info('End of five_users() fixture.')


@pytest.fixture(scope='function')
def ten_users():
    number_of_users = 10
    logger.info('Start of ten_users() fixture.')
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_with_confirmation()
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    create_multiple_auth0_users_and_confirm(number_of_users)
    pause_and_confirm_total_auth0_users(DELAY, number_of_users)
    yield
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_with_confirmation()
    logger.info('End of ten_users() fixture.')


@pytest.fixture(scope='function')
def with_33_auth0_users():
    number_of_users = 33
    logger.info('Start of ten_users() fixture.')
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_with_confirmation()
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    create_multiple_auth0_users_and_confirm(number_of_users)
    pause_and_confirm_total_auth0_users(DELAY, number_of_users)
    yield
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_with_confirmation()
    logger.info('End of ten_users() fixture.')


@pytest.fixture(scope='function')
def with_100_auth0_users():
    number_of_users = 100
    logger.info('Start of ten_users() fixture.')
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_with_confirmation()
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    create_multiple_auth0_users_and_confirm(number_of_users)
    pause_and_confirm_total_auth0_users(DELAY, number_of_users)
    yield
    logger.info('Deleting Django User objects.')
    Auth0User.objects.all().delete()
    logger.info('Deleting all auth0 users.')
    delete_all_auth0_users_with_confirmation()
    logger.info('End of ten_users() fixture.')



from __future__ import unicode_literals

from django.conf import settings
from django.contrib.auth.models import AbstractUser
from django.contrib.auth.validators import UnicodeUsernameValidator
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _

from .validators import Auth0UserIdValidator


class AbstractAuth0User(AbstractUser):
    """
    An abstract base user designed for easy use with Auth0
    """

    # In order to accept the "|" character in a Django Username we need to change the validator.
    use_auth0_username = getattr(settings, 'SOCIAL_AUTH_AUTH0_USER_ID_IS_DJANGO_USERNAME', True)

    if use_auth0_username:
        username_validator = Auth0UserIdValidator()
        username_help_text = _('Required. 150 characters or fewer. Letters, digits and @/./+/-/_/| only.')
    else:
        username_validator = UnicodeUsernameValidator()
        username_help_text = _('Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.')

    username = models.CharField(
        _('username'),
        max_length=150,
        unique=True,
        help_text=username_help_text,
        validators=[username_validator],
        error_messages={
            'unique': _("A user with that username already exists."),
        },
    )

    # ----------------
    # Helper Functions
    # ----------------

    @property
    def auth0_data(self):
        if self.social_auth.count() == 1:
            return self.social_auth.get().extra_data
        else:
            raise NotImplementedError(
                "More than one social auth model instance is associated with this django user model instance"
            )

    # -------------------------------------
    # Convenience / Quick Access Properties
    # -------------------------------------
    # Accessing user data to make decisions should be quick and easy,
    # so our abstract class adds these properties to help make it easier
    # to leverage Auth0 functionality such as the user and app metadata.

    @property
    def user_metadata(self):
        # TODO: Only do this is we are dealing with an OIDC compliant endpoint.
        if self.namespaced_user_metadata_dict_key:
            return self.auth0_data['id_token_payload'][self.namespaced_user_metadata_dict_key]
        return self.auth0_data['id_token_payload']['user_metadata']

    @property
    def app_metadata(self):
        # TODO: Only do this is we are dealing with an OIDC compliant endpoint.
        if self.namespaced_app_metadata_dict_key:
            return self.auth0_data['id_token_payload'][self.namespaced_app_metadata_dict_key]
        return self.auth0_data['id_token_payload']['app_metadata']

    class Meta:
        abstract = True


# TODO: Move this into tests, or raise a warning when used?
# Users of this library should have their own custom user model!



# -*- coding: utf-8
from __future__ import unicode_literals, absolute_import

import django
import os
import environ
import logging.config


DEBUG = True
INTERNAL_IPS = ('127.0.0.1')
USE_TZ = True

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "__________________________________________________"


DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "test.db",
    }
}

ROOT_URLCONF = "tests.urls"

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sites",
    'django.contrib.staticfiles',
    'django.contrib.messages',
    "django.contrib.sessions",
    'social_django',
    "django_auth0_user",
    "tests",
    'test_app',
    'rest_framework',
    'django_extensions',
    'debug_toolbar',
]


BASE_DIR = os.path.dirname(os.path.dirname(__file__))
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, "templates"), ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'social_django.context_processors.backends',
                'social_django.context_processors.login_redirect',
            ],
        },
    },
]

STATIC_URL = '/static/'
STATICFILES_DIRS = [
    os.path.join(BASE_DIR, "static"),
]


# AUTH_TEMPLATES = [{
#     'BACKEND': 'django.template.backends.django.DjangoTemplates',
#     'DIRS': [os.path.join(os.path.dirname(__file__), 'templates')],
#     'APP_DIRS': True,
#     'OPTIONS': {
#         'context_processors': [
#             'django.contrib.auth.context_processors.auth',
#             'django.contrib.messages.context_processors.messages',
#             'social_django.context_processors.backends',
#             'social_django.context_processors.login_redirect',
#         ],
#     },
# }]

SITE_ID = 1

MIDDLEWARE_TUPLE = (
    'debug_toolbar.middleware.DebugToolbarMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

if django.VERSION >= (1, 10):
    MIDDLEWARE = MIDDLEWARE_TUPLE
else:
    MIDDLEWARE_CLASSES = MIDDLEWARE_TUPLE


# AUTH_USER_MODEL = 'auth.User'
AUTH_USER_MODEL = 'test_app.Auth0User'


LOGOUT_REDIRECT_URL = '/'

env = environ.Env()
env.read_env(environ.Path(__file__, is_file=True)('.env'))
AUTH0_DOMAIN = env('AUTH0_USER_DOMAIN')
AUTH0_OIDC_URL = 'https://{}'.format(AUTH0_DOMAIN)
AUTH0_MANAGEMENT_API_CLIENT_ID = env('AUTH0_MANAGEMENT_API_CLIENT_ID')
AUTH0_MANAGEMENT_API_CLIENT_SECRET = env('AUTH0_MANAGEMENT_API_CLIENT_SECRET')
CLEAN_USERNAME_FUNCTION = 'django_auth0_user.clean.slugify'


AUTHENTICATION_BACKENDS = (
    'django_auth0_user.backend.Auth0OpenId',
    'django.contrib.auth.backends.ModelBackend',
)


SOCIAL_AUTH_AUTHENTICATION_BACKENDS = [
    'django_auth0_user.backend.Auth0OpenId'
]

SOCIAL_AUTH_AUTH0_KEY = env('AUTH0_WEB_SITE_CLIENT_ID')
SOCIAL_AUTH_AUTH0_SECRET = env('AUTH0_WEB_SITE_CLIENT_SECRET')
# SOCIAL_AUTH_AUTH0_USER_ID_IS_DJANGO_USERNAME = True


# SOCIAL_AUTH_AUTH0_IGNORE_DEFAULT_SCOPE = True
# SOCIAL_AUTH_AUTH0_SCOPE = ['openid', 'profile', 'email']


# # SOCIAL AUTH AUTH0 BACKEND CONFIG
# SOCIAL_AUTH_TRAILING_SLASH = False
# SOCIAL_AUTH_AUTH0_KEY = os.environ.get('AUTH0_CLIENT_ID')
# SOCIAL_AUTH_AUTH0_SECRET = os.environ.get('AUTH0_CLIENT_SECRET')
# SOCIAL_AUTH_AUTH0_SCOPE = [
#     'openid',
#     'profile'
# ]
# SOCIAL_AUTH_AUTH0_DOMAIN = os.environ.get('AUTH0_DOMAIN')
# AUDIENCE = os.environ.get('AUTH0_AUDIENCE')
# if AUDIENCE:
#     SOCIAL_AUTH_AUTH0_AUTH_EXTRA_ARGUMENTS = {'audience': AUDIENCE}
# AUTHENTICATION_BACKENDS = {
#     'auth0login.auth0backend.Auth0',
#     'django.contrib.auth.backends.ModelBackend'
# }
#
LOGIN_URL = "/login/auth0/"


# ---------------------------------
# Django Rest Framework Stuff
# ---------------------------------

REST_FRAMEWORK = {
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.IsAuthenticated',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'django_auth0_user.drf_authentication.SocialAuthentication',
    ),
}

# ---------------------------------
# Logging
# ---------------------------------

LOGGING_CONFIG = None
LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse',
        },
        'require_debug_true': {
            '()': 'django.utils.log.RequireDebugTrue',
        },
    },
    'handlers': {
        'console':{
            'level': 'INFO',
            'filters': ['require_debug_true'],
            'class': 'logging.StreamHandler',
        },
        'null': {
            'class': 'logging.NullHandler',
        },
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        '': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True
        },
        'django': {
            'handlers': ['console'],
        },
        'django.db.backends.schema': {
            'handlers': ['null'],
            'level': 'WARNING',
            'propagate': False,
        },
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': False,
        },
        'py.warnings': {
            'handlers': ['console'],
        },
        'requests': {
            # The requests library is too verbose in it's logging, reducing the verbosity in our logs.
            'handlers': ['null'],
            'level': 'WARNING',
        },
    }
}

logging.config.dictConfig(LOGGING)




import math

from auth0.v3.authentication import GetToken
from auth0.v3.management import Auth0
from django.conf import settings
from cached_property import threaded_cached_property_with_ttl
import logging


logger = logging.getLogger(__name__)


AUTH0_MANAGEMENT_API_TOKEN_DEFAULT_EXPIRY = 86400  # 24 hours = 86400 seconds

AUTH0_CACHED_TOKEN_TTL = 3600  # 1 hour = 3600 seconds

AUTH0_DOMAIN = getattr(settings, 'AUTH0_DOMAIN')

AUTH0_API_URL = 'https://' + AUTH0_DOMAIN + '/api/v2/'
# domain = AUTH0_DOMAIN

# Non Interactive API Client ID
AUTH0_MANAGEMENT_API_CLIENT_ID = getattr(settings, 'AUTH0_MANAGEMENT_API_CLIENT_ID')
# non_interactive_client_id = 'exampleid'

# Non Interactive API Client Secret
AUTH0_MANAGEMENT_API_CLIENT_SECRET = getattr(settings, 'AUTH0_MANAGEMENT_API_CLIENT_SECRET')
# non_interactive_client_secret = 'examplesecret'

# TODO: handle empty domain, client ID and client secret, in a user friendly fashion.

# get_token = GetToken(domain)
# token = get_token.client_credentials(non_interactive_client_id,
# non_interactive_client_secret, 'https://myaccount.auth0.com/api/v2/')
# mgmt_api_token = token['access_token']


class TokenCache(object):
    """
    Cache the Auth0 token with a TTL so that we get a new one regularly.

    Needed to prevent invalid token issues in production, since the production server needs to possibly live longer
    the token expiry, and we cannot just hammer the token API for a new token every time it makes a request.
    """

    @threaded_cached_property_with_ttl(ttl=AUTH0_CACHED_TOKEN_TTL)
    def auth0_management_api_token(self):
        # TODO: Convert this to a proper logging statement.
        logger.info('No cached Auth0 management API token was found...')
        get_token = GetToken(AUTH0_DOMAIN)
        token = get_token.client_credentials(
            AUTH0_MANAGEMENT_API_CLIENT_ID,
            AUTH0_MANAGEMENT_API_CLIENT_SECRET,
            AUTH0_API_URL
        )
        management_api_token = token['access_token']
        # TODO: Convert this to a proper logging statement.
        logger.info('Successfully generated a new Auth0 Management API Token, this will be cached.')
        return management_api_token


AUTH0_TOKEN_CACHE = TokenCache()


# TODO: Is this the best name for this function?
def get_auth0():
    return Auth0(AUTH0_DOMAIN, AUTH0_TOKEN_CACHE.auth0_management_api_token)


def get_all_auth0_users():
    """
    Update all the user objects...

    :return:
    """
    auth0 = get_auth0()

    auth0_users = []

    first_query = auth0.users.list()
    query_length = first_query['length']
    query_limit = first_query['limit']
    query_start = first_query['start']
    query_total = first_query['total']

    for auth0_user in first_query['users']:
        auth0_users.append(auth0_user)
    if query_length + query_start <= query_total:
        page_number = 0
        while query_length + query_start <= query_total:
            page_number += 1

            page_query = auth0.users.list(page=page_number)

            for auth0_user in page_query['users']:
                auth0_users.append(auth0_user)

            query_length = page_query['length']
            query_limit = page_query['limit']
            query_start = page_query['start']
            query_total = page_query['total']

    return auth0_users


def get_auth0_user(user_id, auth0=None):
    if auth0 is None:
        auth0 = get_auth0()
    # TODO: This does not appear to retrieve all user related data. So it may need to be improved.
    return auth0.users.get(user_id)


def get_users_from_auth0(auth0_conn: Auth0):
    """
    Get all users from Auth0

    :param auth0_conn: Authenticated Auth0 API client
    :param regex: Regex to filter users (re.match(user.email))
    """

    users_list = auth0_conn.users.list()

    total_users = users_list['total']
    page_size = users_list['length']

    # don't waste the first request
    for u in users_list['users']:
        yield u

    del users_list

    # iterate through subsequent pages
    if page_size > 0 and total_users > 0:
        for page in range(1, int(math.ceil(total_users / page_size))):
            for u in auth0_conn.users.list(page=page)['users']:
                yield u


# TODO: Add a function to get an Auth0 client's details from the Management API
# TODO: Add a function that returns true/false based on if the Auth0 client is configured to be OIDC conformant.



# -*- coding: utf-8
from __future__ import unicode_literals, absolute_import
from django.conf.urls import url, include
from django.contrib import admin
from django.conf import settings

from test_app.api_urls import api_v1


urlpatterns = [
    url(r'^api/v1/', include(api_v1.urls)),
    url(r'^admin/', admin.site.urls),
    url(r'^auth/', include('django.contrib.auth.urls')),
    url('', include('social_django.urls', namespace='social')),
    url(r'^', include('test_app.urls')),
]


if settings.DEBUG:
    import debug_toolbar
    urlpatterns.append(
        url(r'^__debug__/', include(debug_toolbar.urls)),
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
test_django-db-auth0-user
------------

Tests for `django-db-auth0-user` models module.
"""
# TODO: Remove Auth0User model & create an example custom user model. Everyone should have their own custom user model.
import logging


logger = logging.getLogger(__name__)


# @pytest.mark.django_db
# @pytest.mark.usefixtures('one_user')
# class TestOneAuth0User(TransactionTestCase):
#     def test_count_one_auth0_user(self):
#         assert Auth0User.objects.count(auth0_db_refresh=True) == 1

    # def test_all_on_one_auth0_user(self):
    #     auth0 = get_auth0()
    #     auth0_users = list(get_users_from_auth0(auth0))
    #     all_users = list(Auth0User.objects.all())
    #     for remote_user in auth0_users:
    #         """For every user in Auth0, check we find them in the list of all users in the DB."""
    #         assert any([remote_user['user_id'] == db_user.auth0_user_id for db_user in all_users])

    # def test_get_one_auth0_user(self):
    #     auth0 = get_auth0()
    #     auth0_users = get_users_from_auth0(auth0)
    #     auth0_user = list(auth0_users)[0]
    #     auth0_user_id = auth0_user['user_id']
    #     user = Auth0User.objects.get(auth0_user_id=auth0_user_id, auth0_db_refresh=True)
    #     assert auth0_user_id == user.auth0_user_id

    # @pytest.mark.xfail()
    # def test_get_one_auth0_user_that_doesnt_exist_should_fail(self):
    #     auth0 = get_auth0()
    #     auth0_users = get_users_from_auth0(auth0)
    #     auth0_user = list(auth0_users)[0]
    #     auth0_user_id = auth0_user['user_id'] + '_not_going_to_work'
    #     with pytest.raises(Auth0User.DoesNotExist):
    #         user = Auth0User.objects.get(auth0_user_id=auth0_user_id, auth0_db_refresh=True)
    #         # assert auth0_user_id == user.auth0_user_id

    # def test_filter_exists_on_one_auth0_user(self):
    #     auth0 = get_auth0()
    #     auth0_users = get_users_from_auth0(auth0)
    #     auth0_user = list(auth0_users)[0]
    #     auth0_user_id = auth0_user['user_id']
    #     assert Auth0User.objects.filter(auth0_user_id=auth0_user_id).exists(auth0_db_refresh=True) is True

    # @pytest.mark.xfail(raises=AssertionError)
    # def test_filter_exists_on_one_auth0_user_that_doesnt_exist_should_fail(self):
    #     auth0 = get_auth0()
    #     auth0_users = get_users_from_auth0(auth0)
    #     auth0_user = list(auth0_users)[0]
    #     auth0_user_id = auth0_user['user_id'] + '_not_going_to_work'
    #     assert Auth0User.objects.filter(auth0_user_id=auth0_user_id).exists() is False


# @pytest.mark.django_db
# @pytest.mark.usefixtures('five_users')
# class TestFiveAuth0Users(TransactionTestCase):
#     def test_count_five_auth0_users(self):
#         assert Auth0User.objects.count() == 5
#
#     def test_all_on_five_auth0_users(self):
#         auth0 = get_auth0()
#         auth0_users = list(get_users_from_auth0(auth0))
#         all_users = list(Auth0User.objects.all())
#         for remote_user in auth0_users:
#             """For every user in Auth0, check we find them in the list of all users in the DB."""
#             assert any([remote_user['user_id'] == db_user.auth0_user_id for db_user in all_users])


# @pytest.mark.django_db
# @pytest.mark.usefixtures('ten_users')
# class TestTenAuth0Users(TransactionTestCase):
#     def test_count_ten_auth0_users(self):
#         assert Auth0User.objects.count() == 10


# @pytest.mark.django_db
# @pytest.mark.usefixtures('with_33_auth0_users')
# class TestThirtyThreeAuth0Users(TransactionTestCase):
#     def test_count_thirty_three_auth0_users(self):
#         assert Auth0User.objects.count() == 33


# @pytest.mark.django_db
# @pytest.mark.usefixtures('with_100_auth0_users')
# class TestOneHundredAuth0Users(TransactionTestCase):
#     def test_count_one_hundred_auth0_users(self):
#         assert Auth0User.objects.count() == 100


# @pytest.mark.django_db
# class TestDjangoDBAuth0User:
#     def test_count_ten_auth0_users(delete_all_auth0_users, delete_all_django_users, create_ten_users_for_auth0_test):
#         assert Auth0User.objects.count() == 10
#
#     def test_count_one_auth0_user(delete_all_auth0_users, delete_all_django_users, create_one_user_for_auth0_test):
#         assert Auth0User.objects.count() == 1
#
#     def test_count_one_hundred_auth0_users(delete_all_auth0_users, delete_all_django_users, create_one_hundred_users_for_auth0_test):
#         assert Auth0User.objects.count() == 100
#
#     @pytest.mark.xfail
#     def test_create_auth0_user_should_fail(self):
#         new_user = Auth0User()
#         new_user.save()

# user_data = {
#         'app_metadata': {},
#         'clientID': 'dHnrsr87JWuA92kc8vV0VC5MCSfrTcUC',
#         'created_at': '2017-08-29T06:31:41.567Z',
#         'email': 'user1@example.com',
#         'email_verified': False,
#         'identities': [{'connection': 'Username-Password-Authentication',
#                  'isSocial': False,
#                  'provider': 'auth0',
#                  'user_id': '59a50a4dfb030d7b2bf0cde7'}],
#         'name': 'user1@example.com',
#         'nickname': 'user1',
#         'picture': 'https://s.gravatar.com/avatar/111d68d06e2d317b5a59c2c6c5bad808?s=480&r=pg&d=https%3A%2F%2Fcdn.auth0.com%2Favatars%2Fus.png',
#         'sub': 'auth0|59a50a4dfb030d7b2bf0cde7',
#         'updated_at': '2017-08-29T08:15:25.378Z',
#         'user_id': 'auth0|59a50a4dfb030d7b2bf0cde7',
#         'user_metadata': {}
#     }
#

# @pytest.mark.django_db
# def test_save_auth0_user():
#     user = Auth0User(**{Auth0User.USERNAME_FIELD: user_data['user_id'], Auth0User.EMAIL_FIELD: user_data['email']})
#     user.save()
#     assert user.is_active
